#!/usr/bin/python3

import sys
import pprint
import json
from datetime import datetime

from web3 import Web3
from web3 import IPCProvider
from web3.middleware import geth_poa_middleware

### GLOBALS ####################################################################

contractAddress = Web3.toChecksumAddress("0x8258f4dea122619198ab9c8a42e7e882cb148396")
contractABI     = [{"constant": False,"inputs": [{"name": "_hash","type": "bytes32"},{"name": "_email","type": "string"},{"name": "_storage","type": "string"}],"name": "doLog","outputs": [],"payable": False,"stateMutability": "nonpayable","type": "function"},{"inputs": [],"payable": False,"stateMutability": "nonpayable","type": "constructor"},{"anonymous": False,"inputs": [{"indexed": True,"name": "_hash","type": "bytes32"},{"indexed": False,"name": "_email","type": "string"},{"indexed": False,"name": "_storage","type": "string"}],"name": "logData","type": "event"}]
provider        = "/home/quadrans/.quadrans/geth.ipc"

### MAIN #######################################################################

def main():
    tx_hash = sys.argv[1]
    web3 = Web3(IPCProvider(provider))
    # web3.middleware_stack.inject(geth_poa_middleware, layer=0)
    web3.middleware_onion.inject(geth_poa_middleware, layer=0)

    contract =  web3.eth.contract( address = contractAddress, abi = contractABI )


    tx_receipt = web3.eth.waitForTransactionReceipt(tx_hash)

    # Decode Data --> Encode JSON
    bl = web3.eth.getBlock(tx_receipt['blockNumber'])
    ts = bl["timestamp"]
    bdt = datetime.utcfromtimestamp(ts).strftime('%Y-%m-%d %H:%M:%S')
    ce = contract.events.logData().processReceipt(tx_receipt)
    res = {
        'blockNumber'       : tx_receipt['blockNumber'],
        'blockHash'         : '0x'+bytes(tx_receipt['blockHash']).hex(),
        'blockDateTime'     : bdt,
        'transactionHash'   : '0x'+bytes(tx_receipt['transactionHash']).hex(),
        'sender'            : tx_receipt['from'],
        'rawData'           : tx_receipt['logs'][0]['data'],
        'data'              : {
            '_hash'     : '0x'+ce[0]['args']['_hash'].hex(),
            '_email'    : ce[0]['args']['_email'],
            '_storage'  : ce[0]['args']['_storage']
        }            
    }

    pprint.pprint(res)

### RUN ########################################################################

if sys.version_info[0] < 3:
    raise Exception("Python 3 or a more recent version is required.")

if __name__ == "__main__": 
    main()
